
import re

from datetime import datetime

from django.conf import settings


def clean_code(text):
    return re.sub(r'[\W_]+', '', text).lower()


def get_date_from_request(request, key):
    return request.GET.get(
        key,
        datetime.now().date().strftime(settings.DATE_INPUT_FORMATS[0])
    )


def get_percent(i, total, max_val=100, decimals=1, ):
    return ("{0:." + str(decimals) + "f}").format(max_val * (i / float(total)))


def print_progress(
        i, total, prefix='Progress', decimals=1, length=100, fill='█'):

    percent = get_percent(i, total, decimals=decimals)

    filled = int(length * i // total)

    bar = fill * filled + '-' * (length - filled)

    print('\r{}: |{}| {}%'.format(prefix, bar, percent), end='\r')

    if i == total:
        print()


def decorate(f, decorators):

    if not decorators:
        return f

    decorated_f = f

    for decorator in reversed(decorators):
        decorated_f = decorator(decorated_f)

    return decorated_f


def get_url_params(*args):

    result = ''

    for arg in args:
        result += '<str:{}_slug>_<int:{}_id>{}'.format(arg[0], arg[0], arg[1])

    return result




from os.path import basename
from urllib.request import urlretrieve

from django.apps import apps
from django.db import models
from django.db.models.base import ModelBase
from django.core.files import File
from django.utils.translation import ugettext_lazy as _

from sorl.thumbnail import get_thumbnail
from ordered_model.models import OrderedModelBase


class ImageRecordMeta(ModelBase):

    def __new__(mcs, name, bases, attrs):

        new_attrs = dict(attrs)

        parent_field = attrs['parent_field']
        parent_model = attrs['parent_model']

        new_attrs[parent_field] = models.ForeignKey(
            parent_model,
            verbose_name=_("Parent"),
            related_name='images',
            on_delete=models.CASCADE,
            null=True,
            blank=True)

        new_attrs['file'] = models.ImageField(
            verbose_name=_("File"),
            upload_to=parent_field + '_images',
            max_length=255)

        new_attrs['order_with_respect_to'] = parent_field

        return super().__new__(mcs, name, bases, new_attrs)


class ImageRecord(OrderedModelBase):

    parent_model = None
    parent_field = ''

    order = models.PositiveIntegerField(_('Ordering'))

    order_field_name = 'order'

    def __str__(self):
        return str(self.parent)

    @property
    def parent(self):
        return getattr(self, self.parent_field)

    @classmethod
    def get_parent_model_class(cls):

        if isinstance(cls.parent_model, str):
            return apps.get_model(cls.parent_model)

        return cls.parent_model

    def get_preview_url(self):
        try:
            return get_thumbnail(self.file.file, '100').url
        except IOError:
            pass

        return ''

    @classmethod
    def create_from_url(cls, url):

        result = urlretrieve(url)

        return cls.objects.create(
            file=File(open(result[0], 'rb'), name=basename(url)))

    class Meta:
        abstract = True
        ordering = ['order']
        verbose_name = _('Image')
        verbose_name_plural = _('Images')


class LogoField(models.ImageField):

    def __init__(
            self,
            verbose_name=_('Logo'),
            upload_to='logos',
            blank=True,
            null=True,
            max_length=255,
            editable=False,
            *args, **kwargs):

        super(LogoField, self).__init__(
            verbose_name=verbose_name,
            upload_to=upload_to,
            blank=blank,
            null=null,
            max_length=max_length,
            editable=editable,
            *args, **kwargs)




from django.shortcuts import render
from django.http.response import Http404
from django.core.exceptions import ObjectDoesNotExist

from basement.utils import decorate


def render_view(template_name, renderer=render, decorators=None):

    def decorator(view_func):

        def wrapper(request, *args, **kwargs):

            try:
                context = decorate(view_func, decorators)(
                    request, *args, **kwargs)
            except ObjectDoesNotExist:
                raise Http404

            return renderer(request, template_name, context)

        return wrapper

    return decorator




from django.apps import apps
from django.urls import path, include
from django.conf import settings
from django.conf.urls.i18n import i18n_patterns
from django.views.generic import RedirectView, TemplateView

from pydoc import locate
from basement import views


def get_urlpatterns(
        home_view=None,
        is_db_download_enabled=False,
        is_i18n_redirect_enabled=True,
        exclude_apps=[]):

    if home_view is None:
        home_view = TemplateView.as_view(template_name='home.html')

    result = [
        path('raise-exception/', lambda request: 1/0)
    ]

    if is_i18n_redirect_enabled:
        result.append(
            path('', RedirectView.as_view(
                url='/{}/'.format(settings.LANGUAGE_CODE))
            )
        )

    result += i18n_patterns(
        path('', home_view, name='home')
    )

    if is_db_download_enabled:
        result.append(
            path('db/download/', views.download_db, name='download-db'))

    if apps.is_installed('ckeditor_uploader'):
        result.append(path('ckeditor/', include('ckeditor_uploader.urls')))

    for app in settings.INSTALLED_APPS:

        if app in exclude_apps:
            continue

        located_urls = locate('{}.urls.app_urls'.format(app))

        if not located_urls:
            continue

        result += located_urls

    return result




from django.conf import settings
from django.forms import DateField
from django.forms import Field, ValidationError
from django.utils.translation import ugettext_lazy as _

from basement.widgets import DatePickerInput


class FormField(Field):

    form = None

    def init_form(self, *args, **kwargs):
        self.form = self._build_form(*args, **kwargs)
        self.widget.form = self.form

    def clean(self, *args, **kwargs):
        if not self.form.is_valid():
            raise ValidationError(_('Form is invalid.'))

        return self.form.cleaned_data

    def commit(self, *args, **kwargs):
        return self.form.commit(*args, **kwargs)

    def _build_form(self, *args, **kwargs):
        raise NotImplementedError


class DatePickerField(DateField):

    widget = DatePickerInput

    input_formats = settings.DATE_INPUT_FORMATS




from basement.views._download_db import download_db
from basement.views._action_view import ActionView
from basement.views._ajax_view import ajax_view
from basement.views._form_action_view import FormActionView
from basement.views._action_view import ActionView
from basement.views._render_view import render_view
from basement.views._form_view import FormView
from basement.views._filter_view import FilterView
from basement.views._delete_view import delete_view




from django.conf import settings
from django.contrib import messages
from django.contrib.auth import views, login, logout, get_user_model
from django.contrib.auth.decorators import login_required
from django.contrib.sites.shortcuts import get_current_site
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from django.shortcuts import redirect, render
from django.urls import reverse_lazy, reverse
from django.core.exceptions import ObjectDoesNotExist
from django.views.generic import FormView, CreateView, UpdateView
from django.utils.decorators import method_decorator
from django.utils.encoding import force_bytes, force_text
from django.utils.http import urlsafe_base64_encode, urlsafe_base64_decode

from accounts import forms
from accounts.tokens import account_activation_token


class LoginView(views.LoginView):

    form_class = forms.LoginForm
    template_name = 'accounts/login.html'

    def form_valid(self, form):

        request = self.request

        login(request, form.get_user())

        session_age = settings.SESSION_COOKIE_AGE \
            if form.cleaned_data.get('remember') else 0

        request.session.set_expiry(session_age)

        message = _('You have successfully logged in as {}')

        messages.success(request, message.format(request.user.username))

        return redirect(self.get_success_url())


class LogoutView(views.LogoutView):

    def dispatch(self, request, *args, **kwargs):

        messages.success(self.request, _('You have successfully logged out'))

        return super().dispatch(request, *args, **kwargs)


class PasswordResetView(views.PasswordResetView):

    subject_template_name = 'accounts/password-reset-subject.txt'
    email_template_name = 'accounts/password-reset-email.html'
    html_email_template_name = 'accounts/password-reset-email.html'
    template_name = 'accounts/password-reset.html'
    success_url = reverse_lazy('home')
    success_message = _('Password reset link was sent to your email address')

    def form_valid(self, form):
        messages.success(self.request, self.success_message)
        return super().form_valid(form)


class PasswordResetConfirmView(views.PasswordResetConfirmView):

    template_name = 'accounts/password-reset-confirm.html'
    success_url = reverse_lazy('home')
    success_message = _('Your password has been changed')

    def form_valid(self, form):
        messages.success(self.request, self.success_message)
        return super().form_valid(form)


class SignupView(CreateView):

    model = get_user_model()
    form_class = forms.SignupForm
    success_url = reverse_lazy('home')
    success_message = _('Email confirmation link was sent to your email')
    template_name = 'accounts/signup.html'
    email_template_name = 'accounts/signup-email.html'
    subject_template_name = 'accounts/signup-subject.txt'

    def form_valid(self, form):

        user = form.save(commit=False)
        user.is_active = False
        user.save()

        request = self.request

        domain = get_current_site(request).domain

        subject = render_to_string(self.subject_template_name, {
            'domain': domain,
            'user': user
        })

        activate_url = reverse('accounts:activate', kwargs={
            'uidb64': (
                urlsafe_base64_encode(force_bytes(user.pk)).decode("utf-8")),
            'token': account_activation_token.make_token(user)
        })

        message = render_to_string(self.email_template_name, {
            'user': user,
            'domain': domain,
            'activate_url': request.build_absolute_uri(activate_url)
        })

        user.email_user(subject, '', html_message=message)

        messages.success(self.request, self.success_message)

        return redirect(self.success_url)


class PasswordChangeView(views.PasswordChangeView):

    template_name = 'accounts/password-change.html'
    success_url = reverse_lazy('accounts:profile')

    success_message = _('Your password has been changed')

    def form_valid(self, form):
        messages.success(self.request, self.success_message)
        return super().form_valid(form)


@method_decorator(login_required, "dispatch")
class ProfileChangeView(UpdateView):

    form_class = forms.UserChangeForm
    success_url = reverse_lazy('accounts:profile')
    template_name = 'accounts/profile-change.html'

    def get_object(self, queryset=None):
        return self.request.user

    def form_valid(self, form):
        form.save()
        messages.success(self.request, _('Profile successfully changed'))
        return super(ProfileChangeView, self).form_valid(form)


@method_decorator(login_required, "dispatch")
class RemoveProfileView(FormView):

    form_class = forms.ConfirmPasswordForm
    success_url = reverse_lazy('home')
    template_name = 'accounts/profile-remove.html'

    def get_form_kwargs(self):
        kwargs = super(RemoveProfileView, self).get_form_kwargs()
        kwargs.update({'user': self.request.user})
        return kwargs

    def form_valid(self, form):
        user = self.request.user
        user.is_active = False
        user.save()

        logout(self.request)

        messages.success(self.request, _('Profile successfully removed'))
        return super(RemoveProfileView, self).form_valid(form)


@login_required
def profile(request):
    return render(request, 'accounts/profile.html')


def activate(request, uidb64, token):

    try:
        uid = force_text(urlsafe_base64_decode(uidb64))
        user = get_user_model().objects.get(pk=uid, is_active=False)
    except (TypeError, ValueError, OverflowError, ObjectDoesNotExist):
        user = None

    if user is not None and account_activation_token.check_token(user, token):

        user.is_active = True
        user.save()

        login(request, user)

        messages.success(request, _('Your account is activated'))

        return redirect('accounts:profile')

    messages.error(request, _('Activation link is invalid'))

    return redirect('home')




from django.forms import Form, ModelForm, ValidationError

from basement.fields import FormField


def get_clean_data(django_form):

    validate_form(django_form)

    return django_form.cleaned_data


def validate_form(django_form):
    if not django_form.is_valid():
        raise ArgumentValidationError(django_form)


class ArgumentValidationError(ValidationError):

    def __init__(self, form):
        self._form = form
        super(ArgumentValidationError, self).__init__(form.errors)

    @property
    def form(self):
        return self._form

    def __str__(self):
        return self.form.errors.as_text()


class BasementFormMixin(object):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for field in self.fields.values():
            if isinstance(field, FormField):
                field.init_form(*args, **kwargs)

    def commit(self, instance):
        for field in self.fields.values():
            if isinstance(field, FormField):
                field.commit(instance)


class BasementForm(BasementFormMixin, Form):
    pass


class BasementModelForm(BasementFormMixin, ModelForm):
    pass





from django.utils.translation import ugettext_lazy as _


class _ExtendedHttpException(Exception):

    def __init__(self, content=None):
        self._content = content
        super(_ExtendedHttpException, self).__init__()

    def __unicode__(self):
        message = 'HTTP error'
        if type(self._content) is dict and self._content.get('message'):
            message = self._content.get('message')
        return message

    def get_content(self):
        return self._content


class UnsupportedMethod(_ExtendedHttpException):

    def __init__(self):
        super(UnsupportedMethod, self).__init__(
            _("Selected http method is not supported."))




from urllib.error import HTTPError

from django.http.response import JsonResponse, HttpResponseBadRequest
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.contenttypes.models import ContentType
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.utils.translation import ugettext_lazy as _

from images.forms import UploadImageForm


@csrf_exempt
@require_POST
@staff_member_required
def upload_image(request, content_type_id):

    form = UploadImageForm(data=request.POST, files=request.FILES)

    ct = ContentType.objects.get_for_id(content_type_id)

    image_model = ct.model_class()

    if form.is_valid():

        data = form.cleaned_data

        if data.get('url'):
            try:
                image = image_model.create_from_url(data['url'])
            except HTTPError as e:
                error = _('Can not upload {}: {}').format(data['url'], str(e))
                return HttpResponseBadRequest(error)

        elif data.get('file'):
            image = image_model.objects.create(file=data['file'])

        else:
            return HttpResponseBadRequest('No data')

        return JsonResponse({'id': image.id, 'url': image.get_preview_url()})

    return HttpResponseBadRequest('Data not valid')




from django.core.files import File


def refresh_logo(instance):

    first_image = instance.images.first()

    if first_image:
        file = File(open(first_image.file.path, 'rb'))
        instance.logo.save(first_image.file.name, file)
    else:
        instance.logo.delete()




from basement import http


class RequestDispatcher(object):

    def __init__(self, views):
        self._views = views

    def __iter__(self):
        for view in self._views:
            yield view

    def __add__(self, other):

        try:
            other_views = list(other)
        except TypeError:
            other_views = [other]

        return RequestDispatcher(self._views + other_views)

    def __call__(self, request, *args, **kwargs):
        for view in self._views:
            try:
                return view(request, *args, **kwargs)
            except http.UnsupportedMethod:
                pass

        raise http.UnsupportedMethod()



from django.template.loader import render_to_string
from django.http.response import HttpResponseBadRequest

from pagination import paginate

from basement import http
from basement.forms import get_clean_data
from basement.views._base_view import BaseView


class FilterView(BaseView):

    def __init__(
            self,
            search_iterator,
            items_template,
            form_class=None,
            per_page=20,
            http_method='GET',
            form_factory=None,
            response_class=http.JsonResponse,
            response_factory=None,
            **kwargs):

        self._http_method = http_method
        self._items_template = items_template
        self._per_page = per_page

        setattr(self, http_method.lower(), self._process_request)

        self._search_iterator = search_iterator

        self._form_class = form_class

        if form_factory is not None:
            self._form_factory = form_factory

        self._response_class = response_class

        if response_factory is not None:
            self._response_factory = response_factory

        super(FilterView, self).__init__(**kwargs)

    def _process_request(self, request, **url_kwargs):

        form = self._get_form(request, url_kwargs)

        if form is None:
            data = {}
        else:
            data=get_clean_data(form)

        try:
            iterator = self._search_iterator(request, url_kwargs, data)
        except Exception as e:
            return HttpResponseBadRequest(str(e))

        return self._get_response(request, iterator)

    def _get_response(self, request, iterator):
        return self._response_factory(
            request=request,
            iterator=iterator,
            response_class=self._response_class
        )

    def _response_factory(self, request, iterator, response_class, **kwargs):

        page = paginate(
            request,
            iterator,
            per_page=self._per_page)

        return response_class({
            'items': render_to_string(self._items_template, {
                'request': request,
                'page_obj': page
            }),
            'has_next': page.has_next(),
            'next_page_url': '{}?{}'.format(
                request.path, page.next_page_number().querystring)
        })

    def _get_form(self, request, url_kwargs):

        if self._form_class:
            return self._form_factory(
                request=request,
                form_class=self._form_class,
                data=http.get_request_data(request),
                url_kwargs=url_kwargs)

        return None

    def _form_factory(self, request, form_class, data, url_kwargs):
        return form_class(data=data)




from django.urls import path, include
from django.conf.urls.i18n import i18n_patterns

from images import views


app_name = 'images'


urlpatterns = [

    path('upload/<int:content_type_id>/', views.upload_image, name='upload')

]


app_urls = i18n_patterns(
    path('images/', include((urlpatterns, app_name))),
)




from basement.fields import FormField

from images.widgets import ImagesFormFieldWidget
from images.forms import build_images_form_class


class ImagesFormField(FormField):

    widget = ImagesFormFieldWidget

    def __init__(self, image_model):
        self._image_model = image_model
        super().__init__()

    def _build_form(self, *args, **kwargs):

        form_class = build_images_form_class(self._image_model)

        return form_class(*args, **kwargs)




from django.conf import settings
from django.forms import DateInput, TextInput
from easy_select2.widgets import Select2Multiple, Select2


class DatePickerInput(DateInput):

    def __init__(self, attrs=None, format=None):

        if format is None:
            format = settings.DATE_INPUT_FORMATS[0]

        if attrs is None:
            attrs = {}

        attrs['data-role'] = 'datepicker'

        super().__init__(attrs=attrs, format=format)


class TagsInput(TextInput):

    def __init__(self, attrs=None):

        if attrs is None:
            attrs = {}

        attrs['data-role'] = 'tagsinput'

        super().__init__(attrs=attrs)


class Select(Select2):
    pass


class SelectMultiple(Select2Multiple):
    pass




from os.path import join

from pydoc import locate

from cbsettings import DjangoDefaults


class BasementSettings(DjangoDefaults):

    BASE_DIR = NotImplemented
    DB_NAME = NotImplemented
    DOMAIN = NotImplemented
    DEV_EMAIL = 'pmaigutyak@gmail.com'

    ROOT_URLCONF = 'core.urls'

    ALLOWED_HOSTS = ['*']

    WSGI_APPLICATION = 'core.wsgi.application'

    TIME_ZONE = 'Europe/Kiev'

    USE_TZ = True

    EMAIL_USE_TLS = True

    SITE_ID = 1

    AUTH_PASSWORD_VALIDATORS = [
        {
            'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
        },
        {
            'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
        },
        {
            'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
        },
        {
            'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
        },
    ]

    SILENCED_SYSTEM_CHECKS = ['mysql.E001', 'mysql.W002']

    CRISPY_TEMPLATE_PACK = 'bootstrap3'

    USE_I18N = True
    USE_L10N = True

    CKEDITOR_UPLOAD_PATH = 'uploads/'

    NOCAPTCHA = True
    RECAPTCHA_PUBLIC_KEY = '6LdHaPsSAAAAAPinOxMD64UtSQtD1J37vp9qjsZw'
    RECAPTCHA_PRIVATE_KEY = '6LdHaPsSAAAAAJRHOT4Edilnp-1xSOqttWNk5dar'

    LANGUAGE_CODE = 'uk'
    LANGUAGES = (('uk', 'UA'), )

    FILE_UPLOAD_PERMISSIONS = 0o755

    THUMBNAIL_QUALITY = 85

    IS_WEBP_ENABLED = False
    IS_LESS_ENABLED = False

    MEDIA_URL = '/media/'
    STATIC_URL = '/static/'

    STATIC_APPS = []

    STATICFILES_STORAGE = 'pipeline.storage.PipelineManifestStorage'

    STATICFILES_FINDERS = DjangoDefaults.STATICFILES_FINDERS + [
        'pipeline.finders.PipelineFinder',
        'djangobower.finders.BowerFinder'
    ]

    @property
    def BOWER_INSTALLED_APPS(self):
        return self._get_static_app_component_list(constants.BOWER_APPS_MAP)

    @property
    def STYLESHEETS(self):
        return self._get_static_app_component_list(constants.APPS_CSS_MAP)

    @property
    def CSS_COMPONENTS(self):
        return {}

    @property
    def JAVASCRIPT(self):
        return self._get_static_app_component_list(constants.APPS_JS_MAP)

    @property
    def JS_COMPONENTS(self):
        return {}

    def _get_static_app_component_list(self, files_map):

        result = []

        for app in self.STATIC_APPS:
            try:
                files = files_map[app]
            except KeyError:
                continue

            if isinstance(files, str):
                result.append(files)
            else:
                result += files

        return tuple(result)

    @property
    def PIPELINE(self):

        stylesheets = {
            'generic': {
                'source_filenames': self.STYLESHEETS,
                'output_filename': 'cache/generic.css',
            }
        }

        for key, filenames in self.CSS_COMPONENTS.items():
            stylesheets[key] = {
                'source_filenames': filenames,
                'output_filename': 'cache/{}.css'.format(key),
            }

        javascript = {
            'generic': {
                'source_filenames': self.JAVASCRIPT,
                'output_filename': 'cache/generic.js'
            }
        }

        for key, filenames in self.JS_COMPONENTS.items():
            javascript[key] = {
                'source_filenames': filenames,
                'output_filename': 'cache/{}.js'.format(key),
            }

        return {
            'JS_COMPRESSOR': 'pipeline.compressors.jsmin.JSMinCompressor',
            'CSS_COMPRESSOR': 'pipeline.compressors.cssmin.CSSMinCompressor',
            'COMPILERS': self.COMPILERS,
            'STYLESHEETS': stylesheets,
            'JAVASCRIPT': javascript
        }

    @property
    def MEDIA_ROOT(self):
        return join(self.BASE_DIR, 'media')

    @property
    def STATIC_ROOT(self):
        return join(self.BASE_DIR, 'static-collect')

    @property
    def BOWER_COMPONENTS_ROOT(self):
        return join(self.BASE_DIR, 'static')

    @property
    def STATICFILES_DIRS(self):
        return [
            join(self.BASE_DIR, 'static'),
            join(self.BASE_DIR, 'static', 'bower_components')
        ]

    @property
    def COMPILERS(self):
        if self.IS_LESS_ENABLED:
            return ['pipeline.compilers.less.LessCompiler']
        return []

    @property
    def LOCALE_PATHS(self):
        return [join(self.BASE_DIR, 'locale')]

    @property
    def EMAIL_SUBJECT_PREFIX(self):
        return '{} |'.format(self.DOMAIN.title())

    @property
    def DEFAULT_FROM_EMAIL(self):
        return '{} <noreply@{}>'.format(self.DOMAIN.upper(), self.DOMAIN)

    @property
    def ADMINS(self):
        return ('Dev', self.DEV_EMAIL),

    @property
    def MANAGERS(self):
        return ('Dev', self.DEV_EMAIL),

    @property
    def MIDDLEWARE(self):
        return [
            'django.middleware.security.SecurityMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.middleware.locale.LocaleMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
            'pipeline.middleware.MinifyHTMLMiddleware',
            'basement.middleware.RequestEnvironmentMiddleware'
        ]

    @property
    def INSTALLED_APPS(self):

        apps = [
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.messages',
            'django.contrib.sites',
            'django.contrib.sitemaps',
            'django.contrib.staticfiles',
            'basement',
            'assets',
            'images',
            'widget_tweaks',
            'pagination',
            'notify',
            'django_cleanup',
            'sorl.thumbnail',
            'pipeline',
            'djangobower',
            'ordered_model',
            'ckeditor',
            'ckeditor_uploader',
            'crispy_forms'
        ]

        if self.USE_I18N:
            apps += ['modeltranslation']

        return apps

    @property
    def DATABASES(self):
        return {
            'default': {
                'ENGINE': 'django.db.backends.postgresql',
                'NAME': self.DB_NAME,
                'USER': 'dev'
            }
        }

    @property
    def TEMPLATES(self):
        return [{
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': self.TEMPLATE_DIRS,
            'APP_DIRS': True,
            'OPTIONS': {
                'context_processors': self.CONTEXT_PROCESSORS
            }
        }]

    @property
    def TEMPLATE_DIRS(self):
        return [
            join(self.BASE_DIR, 'templates')
        ]

    @property
    def CONTEXT_PROCESSORS(self):
        processors = [
            'django.template.context_processors.i18n',
            'django.template.context_processors.debug',
            'django.template.context_processors.request',
            'django.template.context_processors.media',
            'django.template.context_processors.static',
            'django.contrib.auth.context_processors.auth',
            'django.contrib.messages.context_processors.messages',
        ]

        if self.IS_WEBP_ENABLED:
            processors.append('assets.context_processors.webp')

        return processors


class LocalSettingsMixin(object):

    DEBUG = True

    EMAIL_BACKEND = 'basement.email.FileBasedEmailBackend'

    CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.dummy.DummyCache'
        }
    }

    @property
    def EMAIL_FILE_PATH(self):
        return join(self.BASE_DIR, 'tmp')


class ProductionSettingsMixin(object):

    DEBUG = False

    EMAIL_PORT = 587
    EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
    EMAIL_HOST = 'smtp.gmail.com'

    SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')
    SECURE_SSL_REDIRECT = True
    SESSION_COOKIE_SECURE = True
    CSRF_COOKIE_SECURE = True

    @property
    def STATICFILES_DIRS(self):
        return [join(self.BASE_DIR, 'static')]

    @property
    def STATIC_ROOT(self):
        return '/home/dev/sites/{}/public/static'.format(self.DOMAIN)

    @property
    def MEDIA_ROOT(self):
        return '/home/dev/sites/{}/public/media'.format(self.DOMAIN)


def settings_factory(apps):

    cleaned_apps = []

    for setting_class in apps:
        if isinstance(setting_class, str):

            if '.' not in setting_class:
                setting_class += '.settings.default'

            located_class = locate(setting_class)

            if located_class is None:
                raise ValueError('{} not found'.format(setting_class))

            cleaned_apps.append(located_class)
        else:
            cleaned_apps.append(setting_class)

    class CommonSettings(*cleaned_apps):
        pass

    return CommonSettings




from django.urls import path, include
from django.conf.urls.i18n import i18n_patterns

from accounts import views


app_name = 'accounts'


urlpatterns = [

    path('', views.profile, name='profile'),

    path('login/', views.LoginView.as_view(), name='login'),

    path('logout/', views.LogoutView.as_view(), name='logout'),

    path('password/change/', views.PasswordChangeView.as_view(),
         name='password-change'),

    path('password/reset/', views.PasswordResetView.as_view(),
         name='password-reset'),

    path('password/reset/<uidb64>/<token>/',
         views.PasswordResetConfirmView.as_view(),
         name='password-reset-confirm'),

    path('signup/', views.SignupView.as_view(), name='signup'),

    path('activate/<uidb64>/<token>/', views.activate, name='activate'),

    path('change/', views.ProfileChangeView.as_view(), name='profile-change'),

    path('remove/', views.RemoveProfileView.as_view(), name='profile-remove'),

]


app_urls = i18n_patterns(
    path('account/', include((urlpatterns, app_name))),
)




import os

from django.db import models
from django.utils.translation import ugettext_lazy as _


class AbstractImportTask(models.Model):

    created = models.DateTimeField(
        _('Creation date'),
        auto_now_add=True)

    file = models.FileField(
        _('File'),
        upload_to='import_cache',
        blank=True,
        null=True)

    is_completed = models.BooleanField(
        _('Is completed'),
        default=False)

    is_processing = models.BooleanField(_('Is processing'), default=False)

    status = models.CharField(_('Status'), max_length=255, blank=True)

    percent = models.CharField(_('Percent'), max_length=10, blank=True)

    def __str__(self):
        return str(self.created)

    def update_progress(
            self,
            percent,
            status=None,
            is_processing=None,
            is_completed=None):

        update_fields = ['percent']

        self.percent = percent

        if status is not None:
            self.status = status
            update_fields.append('status')

        if is_processing is not None:
            self.is_processing = is_processing
            update_fields.append('is_processing')

        if is_completed is not None:
            self.is_completed = is_completed
            update_fields.append('is_completed')

        self.save(update_fields=update_fields)

    @classmethod
    def get(cls, task_id):
        return cls.objects.get(id=task_id)

    @property
    def filename(self):
        return os.path.basename(self.file.name)

    class Meta:
        abstract = True
        ordering = ['-created']
        verbose_name = _('Import task')
        verbose_name_plural = _('Import tasks')




from functools import wraps

from basement import http
from basement.views._view import View


def ajax_view(http_method='GET', **kwargs):
    def decorator(f):
        return wraps(f)(
            View(
                f,
                http_method,
                response_class=http.JsonResponse,
                **kwargs
            )
        )
    return decorator




_SERVICES = {}


class ServiceLocator(object):

    def __init__(self, **kwargs):
        self._kwargs = kwargs

    def __getattr__(self, name):

        cached_name = '_cached_%s' % name

        try:
            return object.__getattribute__(self, cached_name)
        except AttributeError:
            service = self._resolve_service(name)
            setattr(self, cached_name, service)
            return service

    def _resolve_service(self, name):
        try:
            service_factory = _SERVICES[name]
        except KeyError:
            raise AttributeError(
                "Service '%s' is not available. "
                "Ensure this service is registered by @register_service "
                "decorator. Please note, in order for @register_service "
                "to be invoked at the project initialization stage it must "
                "be used inside the 'models' module of a django app." % name
            )

        return service_factory(services=self, **self._kwargs)


def register_service(name):
    def _decorator(service_factory):
        if name in _SERVICES:
            conflicting_service = _SERVICES[name]
            raise RuntimeError(
                "Cannot register a service under the name '%s'. "
                "This name is already taken by another service %s from %s." %
                (
                    name,
                    repr(conflicting_service),
                    conflicting_service.__module__
                )
            )
        _SERVICES[name] = service_factory
        return service_factory
    return _decorator




from django.views.decorators.http import require_POST
from django.views.decorators.csrf import csrf_exempt


def delete_view(func):

    @csrf_exempt
    @require_POST
    def wrap(*args, **kwargs):
        return func(*args, **kwargs)

    return wrap
